# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted
from past.builtins import basestring

from . import amplpython


class OutputHandler(amplpython.OutputHandler):
    """
    Implement this interface to handle the outputs from the calls to any
    function that causes the underlying AMPL interpreter to display a message.

    Note that errors and warnings are not passed through this interface,
    see :class:`~amplpy.ErrorHandler` for more information.
    """

    def output(self, kind, msg):
        """
         This method is called when AMPL sends some output derived from
         executing a statement.

         Args:
            kind: kind of the output of the AMPL interpreter.

            msg: text of the output by the AMPL interpreter.
        """
        pass


class Kind(object):
    """
    Represents the type of the output coming from the interpreter.
    """

    WAITING = amplpython.WAITING
    """
    Output ``prompt2``, returned when incomplete statements are interpreted.
    """

    BREAK = amplpython.BREAK
    """
    Output ``break``, displayed when an operation is interrupted with SIGINT.
    """

    CD = amplpython.CD
    """
    Output ``cd``, returned by the ``cd`` function.
    """

    DISPLAY = amplpython.DISPLAY
    """
    Output ``display``, returned by the ``display`` function.
    """

    EXIT = amplpython.EXIT
    """
    Output ``exit``, returned as last message from AMPL before exiting the
    interpreter.
    """

    EXPAND = amplpython.EXPAND
    """
    Output ``expand``, returned by the ``expand`` function.

    """

    LOAD = amplpython.LOAD
    """
    Output ``load``, returned by the ``load`` function when loading a library.
    """

    OPTION = amplpython.OPTION
    """
    Output ``option``, returned by the ``option`` function when getting the
    value of an option.
    """

    PRINT = amplpython.PRINT
    """
    Output ``print``, returned by the ``print`` function when printing values
    from AMPL command line.
    """

    PROMPT = amplpython.PROMPT  # prompt1 and prompt3
    """
    Output ``prompt1``, normal AMPL prompt.
    """

    SOLUTION = amplpython.SOLUTION
    """
    Output ``solution``, returned when loading a solution with the command
    ``solution``, contains the solver message.
    """

    SOLVE = amplpython.SOLVE
    """
    Output ``solve``, returned by the ``solve`` function, contains the solver
    message.
    """

    SHOW = amplpython.SHOW
    """
    Output ``show``, returned by the ``show`` function.
    """

    XREF = amplpython.XREF
    """
    Output ``xref``, returned by the ``xref`` function.
    """

    SHELL_OUTPUT = amplpython.SHELL_OUTPUT
    """
    Output of the AMPL command ``shell``.
    """

    SHELL_MESSAGE = amplpython.SHELL_MESSAGE
    """
    Messages from the command ``shell``.
    """

    MISC = amplpython.MISC
    """
    Output ``misc``.
    """

    WRITE_TABLE = amplpython.WRITE_TABLE
    """
    Messages from the command ``write table``.
    """

    READ_TABLE = amplpython.READ_TABLE
    """
    Messages from the command ``read table``.
    """

    _READTABLE = amplpython._READTABLE
    """
    Internal messages from the command ``read table``.
    """

    _WRITETABLE = amplpython._WRITETABLE
    """
    Internal messages from the command ``write table``.
    """

    BREAKPOINT = amplpython.BREAKPOINT
    """
    Breakpoint hit.
    """

    CALL = amplpython.CALL
    """
    Output of a script ``call``.

    """

    CHECK = amplpython.CHECK
    """
    Output of a ``check`` operation.
    """

    CLOSE = amplpython.CLOSE
    """
    Output of a ``close`` command for output redirection.
    """

    COMMANDS = amplpython.COMMANDS
    """
    Output of a ``commands`` call into another file.
    """

    CONTINUE = amplpython.CONTINUE
    """
    Issued when ``continue`` is encountered.
    """

    DATA = amplpython.DATA
    """
    Output of a ``data`` command.
    """

    DELETECMD = amplpython.DELETECMD
    """
    Output of a ``delete`` command.
    """

    DROP = amplpython.DROP
    """
    Output of a ``drop`` command.
    """

    DROP_OR_RESTORE_ALL = amplpython.DROP_OR_RESTORE_ALL
    """
    Internal.
    """

    ELSE = amplpython.ELSE
    """
    Else block.
    """

    ELSE_CHECK = amplpython.ELSE_CHECK
    """
    Internal.
    """

    ENDIF = amplpython.ENDIF
    """
    End of if block.
    """

    ENVIRON = amplpython.ENVIRON
    """
    Output of a ``environ`` command.
    """

    FIX = amplpython.FIX
    """
    Output of a ``fix`` command.
    """

    FOR = amplpython.FOR
    """
    Output of a ``for`` command.
    """

    IF = amplpython.IF
    """
    Output of an ``if`` command.
    """

    LET = amplpython.LET
    """
    Output of a ``let`` command.
    """

    LOOPEND = amplpython.LOOPEND
    """
    End of loop.
    """

    OBJECTIVE = amplpython.OBJECTIVE
    """
    Output of an ``objective`` command.
    """

    OPTION_RESET = amplpython.OPTION_RESET
    """
    Occurs when resetting option values.
    """

    PRINTF = amplpython.PRINTF
    """
    Output of a ``printf`` command.
    """

    PROBLEM = amplpython.PROBLEM
    """
    Output of a ``problem`` command.
    """

    PURGE = amplpython.PURGE
    """
    Output of a ``purge`` command.
    """

    RBRACE = amplpython.RBRACE
    """
    Occurs when a right brace is encountered.
    """

    READ = amplpython.READ
    """
    Output of a ``read`` command.
    """

    RELOAD = amplpython.RELOAD
    """
    Output of a ``reload`` command.
    """

    REMOVE = amplpython.REMOVE
    """
    Output of a ``remove`` command.
    """

    REPEAT = amplpython.REPEAT
    """
    Beginning of a repeat loop.
    """

    REPEAT_END = amplpython.REPEAT_END
    """
    End of a repeat loop.
    """

    RESET = amplpython.RESET
    """
    Output of a ``reset`` command.
    """

    RESTORE = amplpython.RESTORE
    """
    Output of a ``restore`` command.
    """

    RUN_ARGS = amplpython.RUN_ARGS
    """
    Internal.
    """

    SEMICOLON = amplpython.SEMICOLON
    """
    Internal.
    """

    SSTEP = amplpython.SSTEP
    """
    Internal.
    """

    THEN = amplpython.THEN
    """
    Beginning of the ``then`` part of an if statement.
    """

    UNFIX = amplpython.UNFIX
    """
    Output of an ``unfix`` command.
    """

    UNLOAD = amplpython.UNLOAD
    """
    Output of an ``unload`` command.
    """

    UPDATE = amplpython.UPDATE
    """
    Output of an ``update`` command.
    """

    WRITE = amplpython.WRITE
    """
    Output of a ``write`` command.
    """



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted
from past.builtins import basestring

from .iterators import EnvIterator
from .base import BaseClass
from . import amplpython


class Environment(BaseClass):
    """
    This class provides access to the environment variables and provides
    facilities to specify where to load the underlying AMPL interpreter.
    """

    def __init__(self, binaryDirectory=None):
        """
        Constructor with ability to select the location of the AMPL binary.
        Note that if binaryDirectory is set, the automatic lookup for an AMPL
        executable will not be executed.

        Args:
            binaryDirectory: The directory in which look for the AMPL Binary.
        """
        if binaryDirectory is None:
            self._impl = amplpython.Environment()
        else:
            self._impl = amplpython.Environment(binaryDirectory)

    def __iter__(self):
        return EnvIterator(self._impl)

    def __setitem__(self, name, value):
        """
        Add an environment variable to the environment, or change its value if
        already defined.

        Args:
            name: Name of the environment variable.

            value: Value to be assigned.
        """
        self._impl.put(name, value)

    def __getitem__(self, name):
        """
        Searches the current object for an environment variable called name and
        returns an iterator to it if found, otherwise it returns `None`.
        """
        it = self._impl.find(name)
        if it == self._impl.end():
            return None
        else:
            return it.second()

    def setBinDir(self, binaryDirectory):
        """
        Set the location where AMPL API will search for the AMPL executable.

        Args:
            binaryDirectory: The directory in which look for the AMPL Binary.
        """
        self._impl.setBinDir(binaryDirectory)

    def getBinDir(self):
        """
        Get the location where AMPL API will search for the AMPL executable.
        """
        return self._impl.getBinDir()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted
from past.builtins import basestring

from . import TestBase
import unittest
import amplpy
import os


class TestAMPL(TestBase.TestBase):
    """Test AMPL."""

    def testAMPL(self):
        from amplpy import Set, Parameter, Variable, Constraint, Objective
        ampl = self.ampl
        self.assertEqual(ampl.getData('1..3').getNumRows(), 3)
        self.assertEqual(ampl.getData('1..3').getNumCols(), 1)
        # self.assertEqual(ampl.getData('1..3', '1..3').getNumCols(), 2)# FIXME
        ampl.eval('set X := 1..10;')
        self.assertTrue(isinstance(ampl.getEntity('X'), amplpy.Entity))
        self.assertEqual(ampl.getEntity('X').getValues().getNumRows(), 10)
        self.assertEqual(ampl.getData('X').getNumRows(), 10)

        with self.assertRaises(RuntimeError):
            self.assertRaises(ampl.getData('XXX'))
        with self.assertRaises(TypeError):
            self.assertRaises(ampl.getEntity('XXX'))
        with self.assertRaises(TypeError):
            self.assertRaises(ampl.getSet('XXX'))
        with self.assertRaises(TypeError):
            self.assertRaises(ampl.getParameter('XXX'))
        with self.assertRaises(TypeError):
            self.assertRaises(ampl.getVariable('XXX'))
        with self.assertRaises(TypeError):
            self.assertRaises(ampl.getConstraint('XXX'))
        with self.assertRaises(TypeError):
            self.assertRaises(ampl.getObjective('XXX'))
        ampl.reset()  # FIXME: seems to have no efect after eval()
        ampl.eval('set _s; param _p; var _v; s.t. _c: _v = 0; maximize _o: 0;')
        self.assertTrue(isinstance(ampl.getSet('_s'), Set))
        self.assertTrue(isinstance(ampl.getParameter('_p'), Parameter))
        self.assertTrue(isinstance(ampl.getVariable('_v'), Variable))
        self.assertTrue(isinstance(ampl.getConstraint('_c'), Constraint))
        self.assertTrue(isinstance(ampl.getObjective('_o'), Objective))
        print(list(ampl.getSets()))
        # self.assertEqual(len(ampl.getSets()), 1) # FIXME: 2 != 1
        self.assertEqual(len(ampl.getParameters()), 1)
        self.assertEqual(len(ampl.getVariables()), 1)
        self.assertEqual(len(ampl.getConstraints()), 1)
        self.assertEqual(len(ampl.getObjectives()), 1)
        ampl.reset()
        with self.assertRaises(RuntimeError):
            ampl.eval('X')
        self.assertTrue(ampl.isRunning())
        self.assertFalse(ampl.isBusy())

    def testPath(self):
        ampl = self.ampl
        self.assertEqual(
            os.path.abspath(ampl.cd()),
            os.path.abspath(os.curdir)
        )
        ampl.cd('..')
        self.assertEqual(
            os.path.abspath(ampl.cd()),
            os.path.abspath(os.path.join(os.curdir, '..'))
        )

    def testOptions(self):
        ampl = self.ampl
        ampl.setOption('a', 's')
        ampl.setOption('b', 123)
        ampl.setOption('c', 1.23)
        ampl.setOption('d', True)
        with self.assertRaises(TypeError):
            ampl.setOption('e', None)
        self.assertEqual(ampl.getOption('a'), 's')
        self.assertEqual(ampl.getOption('b'), 123)
        self.assertEqual(ampl.getOption('c'), 1.23)
        self.assertEqual(ampl.getOption('d'), True)

    def testHandlers(self):
        from time import sleep
        ampl = self.ampl

        class MyOutputHandler(amplpy.OutputHandler):
            def __init__(self):
                self.lastmsg = None

            def output(self, kind, msg):
                if kind == amplpy.Kind.DISPLAY:
                    print('Display: |{}|'.format(msg))
                self.lastmsg = msg

        class MyErrorHandler(amplpy.ErrorHandler):
            def __init__(self):
                self.lastError = None
                self.lastWarning = None

            def error(self, exception):
                print(type(exception))
                print('Error:', exception.getMessage())
                self.lastError = exception

            def warning(self, exception):
                print(type(exception))
                print('Warning:', exception.getMessage())
                self.lastWarning = exception

        outputHandler = MyOutputHandler()
        ampl.setOutputHandler(outputHandler)
        errorHandler = MyErrorHandler()
        ampl.setErrorHandler(errorHandler)
        self.assertEqual(ampl.getOutputHandler(), outputHandler)
        self.assertEqual(ampl.getErrorHandler(), errorHandler)
        ampl.display('1', '2', '3')
        self.assertTrue('1 = 1' in outputHandler.lastmsg)
        self.assertTrue('2 = 2' in outputHandler.lastmsg)
        self.assertTrue('3 = 3' in outputHandler.lastmsg)
        ampl.eval('display X;')
        self.assertEqual(
            str(errorHandler.lastWarning.getMessage()),
            'X is not defined'
        )
        ampl.eval('diy X;')
        self.assertEqual(
            str(errorHandler.lastError.getMessage()),
            'syntax error'
        )
        self.assertTrue(
            isinstance(errorHandler.lastWarning.getSourceName(), basestring)
        )
        self.assertTrue(
            isinstance(errorHandler.lastWarning.getLineNumber(), int)
        )
        self.assertTrue(
            isinstance(errorHandler.lastWarning.getOffset(), int)
        )
        self.assertTrue(
            isinstance(errorHandler.lastWarning.getMessage(), basestring)
        )

    def testEmptyHandlers(self):
        ampl = self.ampl
        callback = amplpy.Runnable()
        outputHandler = amplpy.OutputHandler()
        errorHandler = amplpy.ErrorHandler()
        ampl.setOutputHandler(outputHandler)
        ampl.setErrorHandler(errorHandler)
        ampl.evalAsync('display 1;', callback)
        ampl.wait()

    def testBrokenHandlers(self):
        ampl = self.ampl

        class MyOutputHandler(amplpy.OutputHandler):
            def output(self, kind, msg):
                assert False

        class ErrorHandlerIgnore(amplpy.ErrorHandler):
            def error(self, exception):
                assert True

            def warning(self, exception):
                assert True

        class ErrorHandlerRaise(amplpy.ErrorHandler):
            def error(self, exception):
                raise exception

            def warning(self, exception):
                raise exception

        with self.assertRaises(RuntimeError):
            ampl.eval('X X;')

        errorHandlerIgnore = ErrorHandlerIgnore()
        ampl.setErrorHandler(errorHandlerIgnore)
        ampl.eval('X X;')

        errorHandlerRaise = ErrorHandlerRaise()
        ampl.setErrorHandler(errorHandlerRaise)
        with self.assertRaises(RuntimeError):
            ampl.eval('X X;')

    def testAsync(self):
        from threading import Lock
        import tempfile
        import shutil

        ampl = self.ampl
        dirpath = tempfile.mkdtemp()

        def str2file(filename, content):
            fullpath = os.path.join(dirpath, filename)
            with open(fullpath, 'w') as f:
                print(content, file=f)
            return fullpath

        class MyOutputHandler(amplpy.OutputHandler):
            def output(self, kind, msg):
                pass

        class MyErrorHandler(amplpy.ErrorHandler):
            def __init__(self, mutex1, mutex2):
                self.mutex1 = mutex1
                self.mutex2 = mutex2

            def error(self, exception):
                try:
                    self.mutex1.release()
                except Exception:
                    pass
                try:
                    self.mutex2.release()
                except Exception:
                    pass
                raise exception

            def warning(self, exception):
                print('Warning:', exception.getMessage())

        class Callback(amplpy.Runnable):
            def __init__(self, mutex1, mutex2):
                self.ready = False
                self.mutex1 = mutex1
                self.mutex2 = mutex2

            def run(self):
                self.mutex2.acquire()
                self.ready = True
                self.mutex1.release()
                self.mutex2.release()

        model = str2file('model.mod', '''
            set X;
            set A := 1..10000000;
            param p{i in A} := i;
        ''')
        data = str2file('data.dat', '''
            set X := 1, 2, 3;
        ''')

        mutex1 = Lock()
        mutex2 = Lock()
        try:
            callback = Callback(mutex1, mutex2)
            outputHandler = MyOutputHandler()
            ampl.setOutputHandler(outputHandler)
            errorHandler = MyErrorHandler(mutex1, mutex2)
            ampl.setErrorHandler(errorHandler)

            mutex1.acquire()
            mutex2.acquire()
            callback.ready = False
            ampl.readAsync(model, callback)
            self.assertFalse(callback.ready)
            mutex2.release()
            mutex1.acquire()
            self.assertTrue(callback.ready)

            mutex2.acquire()
            callback.ready = False
            ampl.readDataAsync(data, callback)
            self.assertFalse(callback.ready)
            mutex2.release()
            mutex1.acquire()
            self.assertTrue(callback.ready)

            mutex2.acquire()
            callback.ready = False
            ampl.evalAsync('display {i in A: i not in A};', callback)
            ampl.interrupt()
            self.assertFalse(callback.ready)
            mutex2.release()
            mutex1.acquire()
            self.assertTrue(callback.ready)
            self.assertFalse(ampl.isBusy())
            self.assertTrue(ampl.isRunning())

        except Exception:
            mutex1.acquire(False)
            mutex1.release()
            mutex2.acquire(False)
            mutex2.release()
            raise

        shutil.rmtree(dirpath)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted
from past.builtins import basestring

from threading import Thread, Lock
from .errorhandler import ErrorHandler
from .outputhandler import OutputHandler
from .objective import Objective
from .variable import Variable
from .constraint import Constraint
from .set import Set
from .parameter import Parameter
from .dataframe import DataFrame
from .iterators import EntityMap
from .exceptions import AMPLException
from .entity import Entity
from .utils import Utils, lock_and_call
from . import amplpython


class DefaultOutputHandler(OutputHandler):
    def output(self, kind, msg):
        print(msg, end='')


class AMPL(object):
    """An AMPL translator.

    An object of this class can be used to do the following tasks:

    - Run AMPL code. See :func:`~amplpy.AMPL.eval` and
      :func:`~amplpy.AMPL.evalAsync`.
    - Read models and data from files. See :func:`~amplpy.AMPL.read`,
      :func:`~amplpy.AMPL.readData`, :func:`~amplpy.AMPL.readAsync`, and
      :func:`~amplpy.AMPL.readDataAsync`.
    - Solve optimization problems constructed from model and data (see
      :func:`~amplpy.AMPL.solve` and :func:`~amplpy.AMPL.solveAsync`).
    - Access single Elements of an optimization problem. See
      :func:`~amplpy.AMPL.getVariable`, :func:`~amplpy.AMPL.getConstraint`,
      :func:`~amplpy.AMPL.getObjective`, :func:`~amplpy.AMPL.getSet`,
      and :func:`~amplpy.AMPL.getParameter`.
    - Access lists of available entities of an optimization problem. See
      :func:`~amplpy.AMPL.getVariables`, :func:`~amplpy.AMPL.getConstraints`,
      :func:`~amplpy.AMPL.getObjectives`, :func:`~amplpy.AMPL.getSets`,
      and :func:`~amplpy.AMPL.getParameters`.

    Error handling is two-faced:

    - Errors coming from the underlying AMPL translator (e.g. syntax errors and
      warnings obtained calling the eval method) are handled by
      the :class:`~amplpy.ErrorHandler` which can be set and get via
      :func:`~amplpy.AMPL.getErrorHandler` and
      :func:`~amplpy.AMPL.setErrorHandler`.
    - Generic errors coming from misusing the API, which are detected in
      Python, are thrown as exceptions.

    The default implementation of the error handler throws exceptions on errors
    and prints to console on warnings.

    The output of every user interaction with the underlying translator is
    handled implementing the abstract class :class:`~amplpy.OutputHandler`.
    The (only) method is called at each block of output from the translator.
    The current output handler can be accessed and set via
    :func:`~amplpy.AMPL.getOutputHandler` and
    :func:`~amplpy.AMPL.setOutputHandler`.
    """

    def __init__(self, environment=None):
        """
        Constructor:
        creates a new AMPL instance with the specified environment if provided.

        Args:
            environment (:class:`~amplpy.Environment`): This allows the user to
            specify the location of the AMPL binaries to be used and to modify
            the environment variables in which the AMPL interpreter will run.

        Raises:
            RuntimeError: If no valid AMPL license has been found or if the
            translator cannot be started for any other reason.
        """
        if environment is None:
            try:
                self._impl = amplpython.AMPL()
            except RuntimeError as e:
                from sys import stderr
                if str(e).startswith('AMPL could not be started'):
                    message = (
                        '''* Please make sure that the AMPL folder is in the'''
                        ''' system search path. *'''
                    )
                    print('*' * len(message))
                    print(message)
                    print('*' * len(message))
                raise
        else:
            self._impl = amplpython.AMPL(environment._impl)
        self._errorhandler = None
        self._outputhandler = None
        self._lock = Lock()
        self.setOutputHandler(DefaultOutputHandler())

    def __del__(self):
        """
        Default destructor:
        releases all the resources related to the AMPL instance (most notably
        kills the underlying  interpreter).
        """
        self.close()

    def getData(self, *statements):
        """
        Get the data corresponding to the display statements. The statements
        can be AMPL expressions, or entities. It captures the equivalent of the
        command:

        .. code-block:: ampl

            display ds1, ..., dsn;

        where ds1, ..., dsn are the ``displayStatements`` with which the
        function is called.

        As only one DataFrame is returned, the operation will fail if the
        results of the display statements cannot be indexed over the same set.
        As a result, any attempt to get data from more than one set, or to get
        data for multiple parameters with a different number of indexing sets
        will fail.

        Args:
            statements: The display statements to be fetched.

        Raises:
            RuntimeError: if the AMPL visualization command does not succeed
            for one of the reasons listed above.

        Returns:
            DataFrame capturing the output of the display
            command in tabular form.
        """
        # FIXME: only works for the first statement.
        return lock_and_call(
            lambda: DataFrame._fromDataFrameRef(
                self._impl.getData(list(statements), len(statements))
            ),
            self._lock
        )

    def getEntity(self, name):
        """
        Get entity corresponding to the specified name (looks for it in all
        types of entities).

        Args:
            name: Name of the entity.

        Raises:
            TypeError: if the specified entity does not exist.

        Returns:
            The AMPL entity with the specified name.
        """
        return lock_and_call(
            lambda: Entity(self._impl.getEntity(name)),
            self._lock
        )

    def getVariable(self, name):
        """
        Get the variable with the corresponding name.

        Args:
            name: Name of the variable to be found.

        Raises:
            TypeError: if the specified variable does not exist.
        """
        return lock_and_call(
            lambda: Variable(self._impl.getVariable(name)),
            self._lock
        )

    def getConstraint(self, name):
        """
        Get the constraint with the corresponding name.

        Args:
            name: Name of the constraint to be found.

        Raises:
            TypeError: if the specified constraint does not exist.
        """
        return lock_and_call(
            lambda: Constraint(self._impl.getConstraint(name)),
            self._lock
        )

    def getObjective(self, name):
        """
         Get the objective with the corresponding name.

         Args:
            name: Name of the objective to be found.

        Raises:
            TypeError: if the specified objective does not exist.
        """
        return lock_and_call(
            lambda: Objective(self._impl.getObjective(name)),
            self._lock
        )

    def getSet(self, name):
        """
        Get the set with the corresponding name.

        Args:
            name: Name of the set to be found.

        Raises:
            TypeError: if the specified set does not exist.
        """
        return lock_and_call(
            lambda: Set(self._impl.getSet(name)),
            self._lock
        )

    def getParameter(self, name):
        """
        Get the parameter with the corresponding name.

        Args:
            name: Name of the parameter to be found.

        Raises:
            TypeError: if the specified parameter does not exist.
        """
        return lock_and_call(
            lambda: Parameter(self._impl.getParameter(name)),
            self._lock
        )

    def eval(self, amplstatements):
        """
        Parses AMPL code and evaluates it as a possibly empty sequence of AMPL
        declarations and statements.

        As a side effect, it invalidates all entities (as the passed statements
        can contain any arbitrary command); the lists of entities will be
        re-populated lazily (at first access)

        The output of interpreting the statements is passed to the current
        OutputHandler (see getOutputHandler and
        setOutputHandler).

        By default, errors are reported as exceptions and warnings are printed
        on stdout. This behavior can be changed reassigning an ErrorHandler
        using setErrorHandler.

        Args:
          amplstatements: A collection of AMPL statements and declarations to
          be passed to the interpreter.

        Raises:
          RuntimeError: if the input is not a complete AMPL statement (e.g.
          if it does not end with semicolon) or if the underlying
          interpreter is not running.
        """
        lock_and_call(
            lambda: self._impl.eval(amplstatements),
            self._lock
        )

    def reset(self):
        """
        Clears all entities in the underlying AMPL interpreter, clears all maps
        and invalidates all entities.
        """
        # self._impl.reset()  # FIXME: causes Segmentation fault
        self.eval('reset;')

    def close(self):
        """
        Stops the underlying engine, and release all any further attempt to
        execute optimization commands without restarting it will throw an
        exception.
        """
        try:
            self._impl.close()
        except AttributeError:
            pass

    def isRunning(self):
        """
        Returns true if the underlying engine is running.
        """
        return lock_and_call(
            lambda: self._impl.isRunning(),
            self._lock
        )

    def isBusy(self):
        """
        Returns true if the underlying engine is doing an async operation.
        """
        # return self._impl.isBusy()
        if self._lock.acquire(False):
            self._lock.release()
            return False
        else:
            return True

    def solve(self):
        """
        Solve the current model.

        Raises:
            RuntimeError: if the underlying interpreter is not running.
        """
        return lock_and_call(
            lambda: self._impl.solve(),
            self._lock
        )

    def readAsync(self, fileName, callback):
        """
        Interprets the specified file asynchronously, interpreting it as a
        model or a script file. As a side effect, it invalidates all entities
        (as the passed file can contain any arbitrary command); the lists of
        entities will be re-populated lazily (at first access).

        Args:
            fileName: Path to the file (Relative to the current working
            directory or absolute).

            callback: Callback to be executed when the file has been
            interpreted.
        """
        def async():
            self._lock.acquire()
            try:
                self._impl.read(fileName)
            except Exception:
                self._lock.release()
                raise
            else:
                self._lock.release()
                callback.run()
        Thread(target=async).start()

    def readDataAsync(self, fileName, callback):
        """
        Interprets the specified data file asynchronously. When interpreting is
        over, the specified callback is called. The file is interpreted as
        data. As a side effect, it invalidates all entities (as the passed file
        can contain any arbitrary command); the lists of entities will be
        re-populated lazily (at first access)

        Args:
            fileName: Full path to the file.

            callback: Callback to be executed when the file has been
            interpreted.
        """
        def async():
            self._lock.acquire()
            try:
                self._impl.readData(fileName)
            except Exception:
                self._lock.release()
                raise
            else:
                self._lock.release()
                callback.run()
        Thread(target=async).start()

    def evalAsync(self, amplstatements, callback):
        """
        Interpret the given AMPL statement asynchronously.

        Args:
          amplstatements: A collection of AMPL statements and declarations to
          be passed to the interpreter.

          callback: Callback to be executed when the statement has been
          interpreted.

        Raises:
          RuntimeError: if the input is not a complete AMPL statement (e.g.
          if it does not end with semicolon) or if the underlying
          interpreter is not running.
        """
        def async():
            self._lock.acquire()
            try:
                self._impl.eval(amplstatements)
            except Exception:
                self._lock.release()
                raise
            else:
                self._lock.release()
                callback.run()
        Thread(target=async).start()

    def solveAsync(self, callback):
        """
        Solve the current model asynchronously.

        Args:
          callback: Callback to be executed when the solver is done.
        """
        def async():
            self._lock.acquire()
            try:
                self._impl.solve()
            except Exception:
                self._lock.release()
                raise
            else:
                self._lock.release()
                callback.run()
        Thread(target=async).start()

    def wait(self):
        """
        Wait for the current async operation to finish.
        """
        self._lock.acquire()
        self._lock.release()

    def interrupt(self):
        """
        Interrupt an underlying asynchronous operation (execution of AMPL code
        by the AMPL interpreter). An asynchronous operation can be started via
        evalAsync(), solveAsync(), readAsync() and readDataAsync().
        Does nothing if the engine and the solver are idle.
        """
        self._impl.interrupt()

    def cd(self, path=None):
        """
        Get or set the current working directory from the underlying
        interpreter (see https://en.wikipedia.org/wiki/Working_directory).

        Args:
            path: New working directory or None (to display the working
            directory).

        Returns:
            Current working directory.
        """
        if path is None:
            return lock_and_call(
                lambda: self._impl.cd(),
                self._lock
            )
        else:
            return lock_and_call(
                lambda: self._impl.cd(path),
                self._lock
            )

    def setOption(self, name, value):
        """
        Set an AMPL option to a specified value.

        Args:
            name: Name of the option to be set (alphanumeric without spaces).

            value: The value the option must be set to.

        Raises:
            InvalidArgumet: if the option name is not valid.

            TypeError: if the value has an invalid type.
        """
        if isinstance(value, bool):
            lock_and_call(
                lambda: self._impl.setBoolOption(name, value),
                self._lock
            )
        elif isinstance(value, int):
            lock_and_call(
                lambda: self._impl.setIntOption(name, value),
                self._lock
            )
        elif isinstance(value, float):
            lock_and_call(
                lambda: self._impl.setDblOption(name, value),
                self._lock
            )
        elif isinstance(value, basestring):
            lock_and_call(
                lambda: self._impl.setOption(name, value),
                self._lock
            )
        else:
            raise TypeError

    def getOption(self, name):
        """
         Get the current value of the specified option. If the option does not
         exist, returns None.

         Args:
            name: Option name.

        Returns:
            Value of the option.

        Raises:
            InvalidArgumet: if the option name is not valid.
        """
        try:
            value = lock_and_call(
                lambda: self._impl.getOption(name).value(),
                self._lock
            )
        except RuntimeError:
            return None
        else:
            try:
                return int(value)
            except ValueError:
                try:
                    return float(value)
                except ValueError:
                    return value

    def read(self, fileName):
        """
        Interprets the specified file (script or model or mixed) As a side
        effect, it invalidates all entities (as the passed file can contain any
        arbitrary command); the lists of entities will be re-populated lazily
        (at first access).

        Args:
            fileName: Full path to the file.

        Raises:
            RuntimeError: in case the file does not exist.
        """
        lock_and_call(
            lambda: self._impl.read(fileName),
            self._lock
        )

    def readData(self, fileName):
        """
        Interprets the specified file as an AMPL data file. As a side effect,
        it invalidates all entities (as the passed file can contain any
        arbitrary command); the lists of entities will be re-populated lazily
        (at first access). After reading the file, the interpreter is put back
        to "model" mode.

        Args:
            fileName: Full path to the file.

        Raises:
            RuntimeError: in case the file does not exist.
        """
        lock_and_call(
            lambda: self._impl.readData(fileName),
            self._lock
        )

    def getValue(self, scalarExpression):
        """
        Get a scalar value from the underlying AMPL interpreter, as a double or
        a string.

        Args:
            scalarExpression: An AMPL expression which evaluates to a scalar
            value.

        Returns:
            The value of the expression.
        """
        return lock_and_call(
            lambda: Utils.castVariant(self._impl.getValue(scalarExpression)),
            self._lock
        )

    def setData(self, dataFrame, setName=None):
        """
        Assign the data in the dataframe to the AMPL entities with the names
        corresponding to the column names.

        Args:
            dataFrame: The dataframe containing the data to be assigned.

            setName: The name of the set to which the indices values of the
            DataFrame are to be assigned.

        Raises:
            AMPLException: if the data assignment procedure was not successful.
        """
        if setName is None:
            lock_and_call(
                lambda: self._impl.setData(dataFrame._impl),
                self._lock
            )
        else:
            lock_and_call(
                lambda: self._impl.setData(dataFrame._impl, setName),
                self._lock
            )

    def readTable(self, tableName):
        """
        Read the table corresponding to the specified name, equivalent to the
        AMPL statement:

        .. code-block:: ampl

            read table tableName;

        Args:
            tableName: Name of the table to be read.
        """
        lock_and_call(
            lambda: self._impl.readTable(tableName),
            self._lock
        )

    def writeTable(self, tableName):
        """
        Write the table corresponding to the specified name, equivalent to the
        AMPL statement

        .. code-block:: ampl

            write table tableName;

        Args:
            tableName: Name of the table to be written.
        """
        lock_and_call(
            lambda: self._impl.writeTable(tableName),
            self._lock
        )

    def display(self, *amplExpressions):
        """
        Writes on the current OutputHandler the outcome of the AMPL statement.

        .. code-block:: ampl

            display e1, e2, .., en;

        where e1, ..., en are the strings passed to the procedure.

        Args:
            amplExpressions: Expressions to be evaluated.
        """
        exprs = list(map(str, amplExpressions))
        lock_and_call(
            lambda: self._impl.displayLst(exprs, len(exprs)),
            self._lock
        )

    def setOutputHandler(self, outputhandler):
        """
        Sets a new output handler.

        Args:
            outputhandler: The function handling the AMPL output derived from
            interpreting user commands.
        """
        class OutputHandlerInternal(OutputHandler):
            def output(self, kind, msg):
                outputhandler.output(kind, msg)

        self._outputhandler = outputhandler
        self._outputhandler_internal = OutputHandlerInternal()
        lock_and_call(
            lambda: self._impl.setOutputHandler(
                self._outputhandler_internal
            ),
            self._lock
        )

    def setErrorHandler(self, errorhandler):
        """
        Sets a new error handler.

        Args:
            errorhandler: The object handling AMPL errors and warnings.
        """
        class InternalErrorHandler(ErrorHandler):
            def error(self, exception):
                if isinstance(exception, amplpython.AMPLException):
                    exception = AMPLException(exception)
                errorhandler.error(exception)

            def warning(self, exception):
                if isinstance(exception, amplpython.AMPLException):
                    exception = AMPLException(exception)
                errorhandler.warning(exception)

        self._errorhandler = errorhandler
        self._errorhandler_internal = InternalErrorHandler()
        lock_and_call(
            lambda: self._impl.setErrorHandler(self._errorhandler_internal),
            self._lock
        )

    def getOutputHandler(self):
        """
        Get the current output handler.

        Returns:
            The current output handler.
        """
        return self._outputhandler

    def getErrorHandler(self):
        """
        Get the current error handler.

        Returns:
            The current error handler.
        """
        return self._errorhandler

    def getVariables(self):
        """
        Get all the variables declared.
        """
        variables = lock_and_call(
            lambda: self._impl.getVariables(),
            self._lock
        )
        return EntityMap(variables, Variable)

    def getConstraints(self):
        """
        Get all the constraints declared.
        """
        constraints = lock_and_call(
            lambda: self._impl.getConstraints(),
            self._lock
        )
        return EntityMap(constraints, Constraint)

    def getObjectives(self):
        """
        Get all the objectives declared.
        """
        objectives = lock_and_call(
            lambda: self._impl.getObjectives(),
            self._lock
        )
        return EntityMap(objectives, Objective)

    def getSets(self):
        """
        Get all the sets declared.
        """
        sets = lock_and_call(
            lambda: self._impl.getSets(),
            self._lock
        )
        return EntityMap(sets, Set)

    def getParameters(self):
        """
        Get all the parameters declared.
        """
        parameters = lock_and_call(
            lambda: self._impl.getParameters(),
            self._lock
        )
        return EntityMap(parameters, Parameter)

    def _var(self):
        """
        Get/Set a variable.
        """
        class Variables(object):
            def __getitem__(_self, name):
                return self.getVariable(name)

            def __setitem__(_self, name, value):
                self.getVariable(name).setValue(value)

            def __iter__(_self):
                return self.getVariables()

        return Variables()

    def _con(self):
        """
        Get/Set a constraint.
        """
        class Constraints(object):
            def __getitem__(_self, name):
                return self.getConstraint(name)

            def __setitem__(_self, name, value):
                self.getConstraint(name).setDual(value)

            def __iter__(_self):
                return self.getConstraints()

        return Constraints()

    def _obj(self):
        """
        Get an objective.
        """
        class Objectives(object):
            def __getitem__(_self, name):
                return self.getObjective(name)

            def __iter__(_self):
                return self.getObjectives()

        return Objectives()

    def _set(self):
        """
        Get/Set a set.
        """
        class Sets(object):
            def __getitem__(_self, name):
                return self.getSet(name)

            def __setitem__(_self, name, values):
                self.getSet(name).setValues(values)

            def __iter__(_self):
                return self.getSets()

        return Sets()

    def _param(self):
        """
        Get/Set a parameter.
        """
        class Parameters(object):
            def __getitem__(_self, name):
                return self.getParameter(name)

            def __setitem__(_self, name, value):
                if isinstance(value, (float, int, basestring)):
                    self.getParameter(name).set(value)
                else:
                    self.getParameter(name).setValues(value)

            def __iter__(_self):
                return self.getParameters()

        return Parameters()

    def _option(self):
        """
        Get/Set an option.
        """
        class Options(object):
            def __getitem__(_self, name):
                return self.getOption(name)

            def __setitem__(_self, name, value):
                self.setOption(name, value)

        return Options()

    var = property(_var)
    con = property(_con)
    obj = property(_obj)
    set = property(_set)
    param = property(_param)
    option = property(_option)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import

import unittest
from .test_ampl import TestAMPL
from .test_entities import TestEntities
from .test_iterators import TestIterators
from .test_dataframe import TestDataFrame
from .test_environment import TestEnvironment
from .test_properties import TestProperties


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted
from past.builtins import basestring


class BaseClass(object):
    def __init__(self, _impl):
        self._impl = _impl

    def toString(self):
        return self._impl.toString()

    def __str__(self):
        return self.toString()



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted
from past.builtins import basestring

from . import amplpython


class AMPLException(amplpython.AMPLException):
    """
    Represent an exception generated by AMPL.
    """
    def __init__(self, _impl):
        self._impl = _impl
        self.sourceName = _impl.getSourceName()
        self.lineNumber = _impl.getLineNumber()
        self.offset = _impl.getOffset()
        self.message = _impl.getMessage()

    def getSourceName(self):
        """
        Get the name of the file where the error was detected.
        """
        return self.sourceName

    def getLineNumber(self):
        """
        Get the row where the error is located.
        """
        return self.lineNumber

    def getOffset(self):
        """
        Get the offset where the error is located.
        """
        return self.offset

    def getMessage(self):
        """
        Get the error message.
        """
        return self.message



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted
from past.builtins import basestring

from .entity import Entity


class Variable(Entity):
    """
    Bases: :class:`~amplpy.Entity`.

    Represents an AMPL decision variable. Note that, in case of a scalar
    variable, all the properties of the variable instance can be accessed
    through methods like :func:`~amplpy.Variable.value`. The methods have the
    same name of the corresponding AMPL suffixes.
    See http://www.ampl.com/NEW/suffbuiltin.html for a list of the available
    suffixes.

    All these methods throw a TypeError if called for a non scalar
    objective and an RuntimeError if called on an entity which has been deleted
    in the underlying intepreter.

    To gain access to all the values in an entity (for all instances and all
    suffixes for that entities), see :func:`~amplpy.Entity.getValues` and the
    :class:`~amplpy.DataFrame` class.
    """

    def __init__(self, _impl):
        Entity.__init__(self, _impl, Variable)

    def value(self):
        """
        Get the current value of this variable.
        """
        return self._impl.value()

    def fix(self, value=None):
        """
        Fix all instances of this variable to a value if provided or to
        their current value otherwise.

        Args:
            value: value to be set.
        """

        if value is None:
            self._impl.fix()
        else:
            self._impl.fix(value)

    def unfix(self):
        """
        Unfix all instances of this variable.
        """
        self._impl.unfix()

    def setValue(self, value):
        """
        Set the current value of this variable (does not fix it),
        equivalent to the AMPL command `let`.

        Args:
            value: value to be set.
        """
        self._impl.setValue(value)

    def astatus(self):
        """
        Get the AMPL status (fixed, presolved, or substituted out).
        """
        return self._impl.astatus()

    def defeqn(self):
        """
        Get the index in `_con` of "defining constraint" used to substitute
        variable out.
        """
        return self._impl.defeqn()

    def dual(self):
        """
        Get the dual value on defining constraint of variable substituted out.
        """
        return self._impl.dual()

    def init(self):
        """
        Get the current initial guess.
        """
        return self._impl.init()

    def init0(self):
        """
        Get the original initial guess (set by `:=` or`default` or by a data
        statement).
        """
        return self._impl.init0()

    def lb(self):
        """
        Returns the current lower bound. See :ref:`secVariableSuffixesNotes`.
        """
        return self._impl.lb()

    def ub(self):
        """
        Returns the current upper bound. See :ref:`secVariableSuffixesNotes`.
        """
        return self._impl.ub()

    def lb0(self):
        """
        Returns the initial lower bounds, from the var declaration.
        """
        return self._impl.lb0()

    def ub0(self):
        """
        Returns the initial upper bound, from the var declaration.
        """
        return self._impl.ub0()

    def lb1(self):
        """
        Returns the weaker lower bound from AMPL's presolve phase.
        """
        return self._impl.lb1()

    def ub1(self):
        """
        Returns the weaker upper bound from AMPL's presolve phase.
        """
        return self._impl.ub1()

    def lb2(self):
        """
        Returns the stronger lower bound from AMPL's presolve phase.
        """
        return self._impl.lb2()

    def ub2(self):
        """
        Returns the stronger upper bound from AMPL's presolve phase.
        """
        return self._impl.ub2()

    def lrc(self):
        """
        Returns the reduced cost at lower bound.
        """
        return self._impl.lrc()

    def urc(self):
        """
        Returns the reduced cost at upper bound.
        """
        return self._impl.urc()

    def lslack(self):
        """
        Return the slack at lower bound (``val - lb``). See
        :ref:`secVariableSuffixesNotes`.
        """
        return self._impl.lslack()

    def uslack(self):
        """
        Return the slack at upper bound (``ub - val``). See
        :ref:`secVariableSuffixesNotes`.
        """
        return self._impl.uslack()

    def rc(self):
        """
        Get the reduced cost (at the nearer bound).
        """
        return self._impl.rc()

    def slack(self):
        """
        Returns the bound slack which is the lesser of
        :func:`~amplpy.Variable.lslack` and :func:`~amplpy.Variable.uslack`.
        See :ref:`secVariableSuffixesNotes`.
        """
        return self._impl.slack()

    def sstatus(self):
        """
        Solver status (basis status of variable).
        """
        return self._impl.sstatus()

    def status(self):
        """
        AMPL status if not `in`, otherwise solver status.
        """
        return self._impl.status()



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from .outputhandler import OutputHandler, Kind
from .errorhandler import ErrorHandler
from .exceptions import AMPLException
from .iterators import EntityMap
from .runnable import Runnable
from .objective import Objective
from .variable import Variable
from .constraint import Constraint
from .set import Set
from .parameter import Parameter
from .entity import Entity
from .dataframe import DataFrame
from .utils import multidict
from .environment import Environment
from .ampl import AMPL
__version__ = '0.3.0'



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted
from past.builtins import basestring

from .entity import Entity


class Objective(Entity):
    """
    Bases: :class:`~amplpy.Entity`.

    Represents an AMPL objective. Note that, in case of a scalar objective, all
    the properties (corresponding to AMPL suffixes) of the objective instance
    can be accessed through methods like :func:`~amplpy.Objective.value`.
    The methods have the same name of the corresponding AMPL suffixes.
    See http://www.ampl.com/NEW/suffbuiltin.html for a list of the available
    suffixes.

    All these methods throw a TypeError if called for a non scalar
    objective and an RuntimeError if called on an entity which has been deleted
    in the underlying intepreter.

    To gain access to all the values in an entity (for all instances and all
    suffixes for that entities), see :func:`~amplpy.Entity.getValues` and the
    :class:`~amplpy.DataFrame` class.
    """

    def __init__(self, _impl):
        Entity.__init__(self, _impl, Objective)

    def value(self):
        """
        Get the value of the objective.
        """
        return self._impl.value()

    def astatus(self):
        """
        Return the AMPL status.
        """
        return self._impl.astatus()

    def sstatus(self):
        """
        Return the solver status.
        """
        return self._impl.sstatus()

    def exitcode(self):
        """
        Exit code returned by solver after most recent solve with this
        objective.
        """
        return self._impl.exitcode()

    def message(self):
        """
        Result message returned by solver after most recent solve with this
        objective.
        """
        return self._impl.message()

    def result(self):
        """
        Result string returned by solver after most recent solve with this
        objective.
        """
        return self._impl.result()

    def drop(self):
        """
        Drop this objective instance.
        """
        self._impl.drop()

    def restore(self):
        """
        Restore this objective (if it had been dropped, no effect otherwise).
        """
        self._impl.restore()

    def minimization(self):
        """
        Get the sense of this objective. Returns True if minimize,
        False if maximize.
        """
        return self._impl.minimization()



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted
from past.builtins import basestring

from .base import BaseClass
from .utils import Utils, Tuple
from .iterators import RowIterator, ColIterator
from . import amplpython


class Row(BaseClass):
    """
    Represents a row in a :class:`~amplpy.DataFrame`.
    """

    def __init__(self, _impl):
        self._impl = _impl

    def __iter__(self):
        return RowIterator(self._impl)

    def __getitem__(self, key):
        return Utils.castVariantRef(self._impl.getIndex(key))

    def toString(self):
        return str(list(self))


class Column(BaseClass):
    """
    Represents a column in a :class:`~amplpy.DataFrame`.
    """

    def __init__(self, _impl):
        self._impl = _impl

    def __iter__(self):
        return ColIterator(self._impl)

    def toString(self):
        return str(list(self))


class DataFrame(BaseClass):
    """
    A DataFrame object, used to communicate data to and from the AMPL entities.

    An object of this class can be used to do the following tasks:
    - Assign values to AMPL entities (once the DataFrame is populated, use
    :func:`~amplpy.AMPL.setData` to assign its values to the modelling entities
    in its columns)
    - Get values from AMPL, decoupling the values from the AMPL entities they
    originate via :func:`~amplpy.Entity.getValues`.

    A DataFrame object can be created in various ways.

    - Create a skeleton by specifiying manually the indexing columns and the
      column headers.
    - Get values from AMPL, decoupling the values from the AMPL entities they
      originate from (via :func:`~amplpy.Entity.getValues`).

    Populating a DataFrame object can be done adding row by row to a
    pre-existing skeleton via :func:`~amplpy.DataFrame.addRow`, setting whole
    columns of a pre-existing skeleton via :func:`~amplpy.DataFrame.setColumn`
    or adding columns (including indexing columns) via
    :func:`~amplpy.DataFrame.addColumn`.

    Modifying a DataFrame object can be done via
    :func:`~amplpy.DataFrame.setColumn` or, item by item, via
    :func:`~amplpy.DataFrame.setValue`.

    Accessing data in a DataFrame can be done row by row using
    :func:`~amplpy.DataFrame.getRow` or by column via
    :func:`~amplpy.DataFrame.getColumn`.
    """

    def __init__(self, index, columns=tuple(), **kwargs):
        """
        Create a new DataFrame with specifed index and column headers.

        Args:
            index: Index column;

            columns: Column headers.
        """
        if index is not None:
            if isinstance(index, basestring):
                index = (index,)
            if isinstance(columns, basestring):
                columns = (columns,)
            index_names = [
                col[0] if isinstance(col, tuple) else col
                for col in index
            ]
            column_names = [
                col[0] if isinstance(col, tuple) else col
                for col in columns
            ]
            self._impl = amplpython.DataFrame.factory(
                len(index_names),
                list(index_names) + list(column_names),
                len(index_names) + len(column_names)
            )
            for col in index:
                if isinstance(col, tuple):
                    self.setColumn(col[0], col[1])
            for col in columns:
                if isinstance(col, tuple):
                    self.setColumn(col[0], col[1])
        else:
            self._impl = kwargs.get('_impl', None)

    def __iter__(self):
        # FIXME: C++ iterators for dataframes not working with SWIG.
        return (self.getRowByIndex(i) for i in range(self.getNumRows()))

    def getNumCols(self):
        """
        Get the total number of columns in this dataframe (indexarity + number
        of values).

        Returns:
            The number of columns.
        """
        return self._impl.getNumCols()

    def getNumRows(self):
        """
        Get the number of data rows in this dataframe.

        Returns:
            The number of rows.
        """
        return self._impl.getNumRows()

    def getNumIndices(self):
        """
        Get the number of indices (the indexarity) of this dataframe.

        Returns:
            The number of indices needed to access one row of this dataframe.
        """
        return self._impl.getNumIndices()

    def addRow(self, *value):
        """
         Add a row to the DataFrame. The size of the tuple must be equal to the
         total number of columns in the dataframe.

         Args:
            value: A single argument with a tuple containing all the values
            for the row to be added, or multiple arguments with the values for
            each column.
        """
        if len(value) == 1 and isinstance(value[0], (tuple, list)):
            value = value[0]
        assert len(value) == self.getNumCols()
        self._impl.addRow(Tuple(value)._impl)

    def addColumn(self, header, values=[]):
        """
        Add a new column with the corresponding header and values to the
        dataframe.

        Args:
            header: The name of the new column.

            values: A list of size :func:`~amplpy.DataFrame.getNumRows` with
            all the values of the new column.
        """
        if len(values) == 0:
            self._impl.addColumn(header)
        else:
            assert len(values) == self.getNumRows()
            if any(isinstance(value, basestring) for value in values):
                values = list(map(str, values))
                self._impl.addColumnStr(header, values)
            elif all(isinstance(value, (float, int)) for value in values):
                values = list(map(float, values))
                self._impl.addColumnDbl(header, values)
            else:
                raise NotImplementedError

    def getColumn(self, header):
        """
        Get the specified column as a view object.

        Args:
            header: The header of the column.
        """
        return Column(self._impl.getColumn(header))

    def setColumn(self, header, values):
        """
        Set the values of a column.

        Args:
            header: The header of the column to be set.

            values: The values to set.
        """
        if any(isinstance(value, basestring) for value in values):
            values = list(map(str, values))
            self._impl.setColumnStr(header, values, len(values))
        elif all(isinstance(value, (float, int)) for value in values):
            values = list(map(float, values))
            self._impl.setColumnDbl(header, values, len(values))
        else:
            raise NotImplementedError

    def getRow(self, key):
        """
        Get a row by value of the indexing columns. If the index is not
        specified, gets the only row of a dataframe with no indexing columns.

        Args:
            key: Tuple representing the index of the desired row.

        Returns:
            The row.
        """
        return Row(self._impl.getRow(Tuple(key)._impl))

    def getRowByIndex(self, index):
        """
        Get row by numeric index.

        Args:
            index: Zero-based index of the row to get.

        Returns:
            The corresponding row.
        """
        assert isinstance(index, int)
        return Row(self._impl.getRowByIndex(index))

    def getHeaders(self):
        """
         Get the headers of this DataFrame.

         Returns:
            The headers of this DataFrame.
        """
        headers = self._impl.getHeaders()
        return tuple(headers.getIndex(i) for i in range(headers.size()))

    def setValues(self, values):
        """
        Set the values of a DataFrame from a dictionary.

        Args:
            values: Dictionary with the values to set.
        """
        ncols = self.getNumCols()
        nindices = self.getNumIndices()
        for key, value in values.items():
            key = Utils.convToList(key)
            assert len(key) == nindices
            value = Utils.convToList(value)
            assert len(value) == ncols-nindices
            self.addRow(key + value)

    def toDict(self):
        """
        Return a dictionary with the DataFrame data.
        """
        d = {}
        nindices = self.getNumIndices()
        for i in range(self.getNumRows()):
            row = list(self.getRowByIndex(i))
            if nindices > 1:
                key = tuple(row[:nindices])
            else:
                key = row[0]
            d[key] = row[nindices:]
        return d

    def toList(self):
        """
        Return a list with the DataFrame data.
        """
        return [tuple(self.getRowByIndex(i)) for i in range(self.getNumRows())]

    def toPandas(self):
        """
        Return a pandas DataFrame with the DataFrame data.
        """
        import pandas as pd
        nindices = self.getNumIndices()
        headers = self.getHeaders()
        columns = {
            header: list(self.getColumn(header))
            for header in headers[nindices:]
        }
        index = zip(*[
            list(self.getColumn(header))
            for header in headers[:nindices]
        ])
        index = [key if len(key) > 1 else key[0] for key in index]
        return pd.DataFrame(columns, index=index)

    @classmethod
    def fromPandas(cls, df):
        """
        Create a :class:`~amplpy.DataFrame` from a pandas DataFrame.
        """
        keys = [
            key if isinstance(key, tuple) else (key,)
            for key in df.index.tolist()
        ]
        index = [
            ('index{}'.format(i), cindex)
            for i, cindex in enumerate(zip(*keys))
        ]
        columns = [
            (cname, df[cname].tolist())
            for cname in df.columns.tolist()
        ]
        return cls(index=index, columns=columns)

    @classmethod
    def _fromDataFrameRef(cls, dfRef):
        return cls(None, None, _impl=dfRef)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted

from . import TestBase
import unittest
from amplpy import DataFrame
import amplpy


class TestDataFrame(TestBase.TestBase):
    """Test DataFrame."""

    def testDataFrame(self):
        ampl = self.ampl
        # Create first dataframe (for data indexed over NUTR)
        # Add data row by row
        df1 = DataFrame('NUTR', ('n_min', 'n_max'))
        df1.addRow(('A', 700, 20000))
        df1.addRow(('B1', 700, 20000))
        df1.addRow(('B2', 700, 20000))
        df1.addRow(('C', 700, 20000))
        df1.addRow(('CAL', 16000, 24000))
        df1.addRow(('NA', 0.0, 50000))

        # Create second dataframe (for data indexed over FOOD)
        # Add column by column
        df2 = DataFrame('FOOD')
        foods = ['BEEF', 'CHK', 'FISH', 'HAM', 'MCH', 'MTL', 'SPG', 'TUR']
        df2.setColumn('FOOD', foods)
        contents = [2] * 8
        df2.addColumn('f_min', contents)
        contents = [10] * 8
        df2.addColumn('f_max', contents)
        costs = [3.19, 2.59, 2.29, 2.89, 1.89, 1.99, 1.99, 2.49]
        df2.addColumn('cost', costs)

        print(df2.getColumn('FOOD'))
        for index in df2.getColumn('FOOD'):
            print(df2.getRow(index))

        # Create third dataframe, to assign data to the AMPL entity
        # param amt{NUTR, FOOD};
        df3 = DataFrame(('NUTR', 'FOOD'))
        # Populate the set columns
        nutrWithMultiplicity = ['']*48
        foodWithMultiplicity = ['']*48
        i = 0
        for n in range(6):
            for f in range(8):
                print(df1.getRowByIndex(n)[0])
                nutrWithMultiplicity[i] = df1.getRowByIndex(n)[0]
                foodWithMultiplicity[i] = foods[f]
                i += 1
        df3.setColumn('NUTR', nutrWithMultiplicity)
        df3.setColumn('FOOD', foodWithMultiplicity)

        # Populate with all these values
        values = [
            60, 8, 8, 40, 15, 70, 25, 60, 10, 20, 15, 35, 15, 15, 25, 15, 15,
            20, 10, 10, 15, 15, 15, 10, 20, 0, 10, 40, 35, 30, 50, 20, 295,
            770, 440, 430, 315, 400, 370, 450, 968, 2180, 945, 278, 1182, 896,
            1329, 1397
        ]
        df3.addColumn('amt', values)

    def testPandas(self):
        try:
            import pandas as pd
        except ImportError:
            return
        df = pd.DataFrame({
            'a': [1, 2],
            'b': [3, 4]
        },
            index=['x', 'y']
        )
        df = DataFrame.fromPandas(df)
        self.assertTrue(isinstance(df.toDict(), dict))
        self.assertTrue(isinstance(df.toList(), list))
        self.assertTrue(isinstance(df.toPandas(), pd.DataFrame))
        self.assertEqual(set(df.toDict().keys()), set(['x', 'y']))
        self.assertEqual(set(df.toList()[0][1:]), set([1, 3]))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted
from past.builtins import basestring

from . import amplpython
from .base import BaseClass


def lock_and_call(call, lock):
    lock.acquire()
    try:
        result = call()
    except Exception:
        lock.release()
        raise
    else:
        lock.release()
        return result


class Tuple(BaseClass):
    def __init__(self, values=None, **kwargs):
        if values is not None:
            values = Utils.convToList(values)
            va = Utils.toVariantArray(values)
            self._impl = amplpython.Tuple.Factory(va, len(values))
        else:
            self._impl = kwargs.get('_impl', None)

    def toTuple(self):
        return tuple(
            Utils.castVariant(self._impl.getIndex(i))
            for i in range(self._impl.size())
        )

    def toPyObj(self):
        size = self._impl.size()
        if size == 1:
            return Utils.castVariant(self._impl.getIndex(0))
        else:
            return self.toTuple()

    @classmethod
    def fromTupleRef(cls, tupleRef):
        return cls(_impl=amplpython.Tuple.Factory(tupleRef))


class Utils(object):
    @staticmethod
    def toVariantArray(lst):
        va = amplpython.VariantArray(len(lst))
        for i in range(len(lst)):
            va[i] = amplpython.Variant(lst[i])
        return va

    @staticmethod
    def toTupleArray(lst):
        ta = amplpython.TupleArray(len(lst))
        for i in range(len(lst)):
            ta[i] = Tuple(lst[i])._impl
        return ta

    @staticmethod
    def castVariant(variant):
        if variant.type() == amplpython.NUMERIC:
            return variant.dbl()
        elif variant.type() == amplpython.STRING:
            return variant.str()

    @staticmethod
    def castStringArray(sarray):
        return [sarray.getIndex(i) for i in range(sarray.size())]

    @staticmethod
    def castVariantRef(variantref):
        return Utils.castVariant(amplpython.Variant(variantref))

    @staticmethod
    def convToList(value):
        if isinstance(value, list):
            return value
        elif isinstance(value, tuple):
            return list(value)
        else:
            return [value]


def multidict(d):
    try:
        ncols = min(len(d[k]) for k in d)
    except TypeError:
        raise TypeError
    return [list(d.keys())] + [{k: d[k][i] for k in d} for i in range(ncols)]



# -*- coding: utf-8 -*-
"""
AMPLPY
------

AMPL API is an interface that allows developers to access the features of the
AMPL interpreter from within a programming language. All model generation and
solver interaction is handled directly by AMPL, which leads to great stability
and speed; the library just acts as an intermediary, and the added overhead
(in terms of memory and CPU usage) depends mostly on how much data is read
back from AMPL, the size of the model as such is irrelevant. Functions for
directly assigning data to AMPL parameters and sets are provided, which can
be used instead of the normal AMPL data reading procedures.  AMPL API has been
written with usability in mind, and it is easy to access its functionalities
from C++, Java, C#, MATLAB and Python.

The AMPL API can function as an add-on to any existing AMPL installation. If
you do not yet have an AMPL installation on the computer where you will be
working with the API, see our
`demo page <http://ampl.com/try-ampl/download-a-free-demo/>`_
or
`trial page <http://ampl.com/try-ampl/request-a-full-trial/>`_
to download a working version that can be installed quickly.

Documentation:
``````````````

* http://amplpy.readthedocs.io
* http://ampl.com/api/nightly/python/

Repositories:
`````````````

* GitHub Repository: https://github.com/ampl/amplpy
* PyPI Repository: https://pypi.python.org/pypi/amplpy
"""
from setuptools import setup, Extension
import platform
import os

OSTYPE = platform.system()
x64 = platform.architecture()[0] == '64bit'


def ls_dir(base_dir):
    """List files recursively."""
    return [
        os.path.join(dirpath.replace(base_dir, '', 1), f)
        for (dirpath, dirnames, files) in os.walk(base_dir)
        for f in files
    ]


def make_relative_rpath(path):
    if OSTYPE == 'Darwin':
        return '-Wl,-rpath,@loader_path/' + path
    elif OSTYPE == 'Linux':
        return '-Wl,-rpath,$ORIGIN/' + path
    else:
        return ''


def compile_args():
    if OSTYPE == 'Windows':
        return ['/TP /EHsc']
    elif OSTYPE == 'Linux':
        return ['-std=c++11']
    else:
        return []


libdir = 'lib64' if x64 else 'lib32'

setup(
    name='amplpy',
    version='0.3.0',
    description='Python API for AMPL',
    long_description=__doc__,
    license='BSD-3',
    platforms='any',
    author='Filipe Brandão',
    author_email='fdabrandao@ampl.com',
    url='http://ampl.com/',
    download_url='https://github.com/ampl/amplpy',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Topic :: Software Development',
        'Topic :: Scientific/Engineering',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: C++',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    packages=['amplpy'],
    ext_modules=[Extension(
        '_amplpython',
        libraries=['ampl'],
        library_dirs=[os.path.join('amplpy', 'amplpython', libdir)],
        include_dirs=[os.path.join('amplpy', 'amplpython', 'include')],
        extra_compile_args=compile_args(),
        extra_link_args=[
            make_relative_rpath(os.path.join('amplpy', 'amplpython', libdir))
        ],
        sources=[
            os.path.join('amplpy', 'amplpython', 'amplpythonPYTHON_wrap.cxx')
        ],
    )],
    package_data={'': ls_dir('amplpy/')},
    install_requires=['future >= 0.15.0']
)



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted
from past.builtins import basestring

from . import amplpython


class ErrorHandler(amplpython.ErrorHandler):
    """
    A basic interface for AMPL error handlers. If an application needs to
    implement customised error handling, it must implement this interface and
    then register an instance with the AMPL API using the
    :func:`~amplpy.AMPL.setErrorHandler` method. The underlying AMPL
    interpreter will then report all errors and warnings through this
    interface as :class:`~amplpy.AMPLException` objects.
    """

    def error(self, amplexception):
        """
        Receives notification of an error.
        """
        pass

    def warning(self, amplexception):
        """
        Receives notification of a warning.
        """
        pass



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted
from past.builtins import basestring

from .entity import Entity
from .utils import Utils, Tuple
from .dataframe import DataFrame


class Parameter(Entity):
    """
    Represents an AMPL parameter. The values can be float or string (in case of
    symbolic parameters).

    Data can be assigned to the set using the methods
    :func:`~amplpy.Parameter.set` and :func:`~amplpy.Parameter.setValues` or
    using :func:`~amplpy.AMPL.setData` and an object of class
    :class:`~amplpy.DataFrame`.
    """

    def __init__(self, _impl):
        Entity.__init__(
            self,
            _impl,
            lambda it: Utils.castVariantRef(it)
        )

    def __setitem__(self, index, value):
        self.set(index, value)

    def isSymbolic(self):
        """
        Returns True if the parameter is declared as symbolic (can store both
        numerical and string values).
        """
        return self._impl.isSymbolic()

    def hasDefault(self):
        """
        Check if the parameter has a default initial value. In case of the
        following AMPL code:

        .. code-block:: ampl

            param a;
            param b default a;

        the function will return true for parameter ``b``.

        Returns:
            True if the parameter has a default initial value. Please note
            that if the parameter has a default expression which refers to
            another parameter which value is not defined, this will return
            True.
        """
        return self._impl.hasDefault()

    def value(self):
        """
        Get the value of this parameter. Valid only for non-indexed parameters.
        """
        return self.get()

    def set(self, *args):
        """
        Set the value of a single instance of this parameter.

        Args:
            args: value if the parameter is scalar, index and value
            otherwise.

        Raises:
            RuntimeError: If the entity has been deleted in the underlying
            AMPL.

            TypeError: If the parameter is not scalar and the index is not
            provided.
        """
        assert len(args) in (1, 2)
        if len(args) == 1:
            value = args[0]
            self._impl.set(value)
        else:
            index, value = args
            if isinstance(value, (float, int)):
                self._impl.setTplDbl(Tuple(index)._impl, value)
            elif isinstance(value, basestring):
                self._impl.setTplStr(Tuple(index)._impl, value)
            else:
                raise TypeError

    def setValues(self, values):
        """
        Assign the values (string or float) to the parameter instances with the
        specified indices, equivalent to the AMPL code:

        .. code-block:: ampl

            let {i in indices} par[i] := values[i];

        Args:
            values: list, dictionary or :class:`~amplpy.DataFrame` with the
            indices and the values to be set.

        Raises:
            TypeError: If called on a scalar parameter.
        """
        if isinstance(values, dict):
            indices, values = list(zip(*values.items()))
            indices = Utils.toTupleArray(indices)
            if any(isinstance(value, basestring) for value in values):
                values = list(map(str, values))
                self._impl.setValuesTaStr(indices, values, len(values))
            elif all(isinstance(value, (float, int)) for value in values):
                values = list(map(float, values))
                self._impl.setValuesTaDbl(indices, values, len(values))
            else:
                raise TypeError
        elif isinstance(values, (list, set)):
            if any(isinstance(value, basestring) for value in values):
                values = list(map(str, values))
                self._impl.setValuesStr(values, len(values))
            elif all(isinstance(value, (float, int)) for value in values):
                values = list(map(float, values))
                self._impl.setValuesDbl(values, len(values))
            else:
                raise TypeError
        else:
            Entity.setValues(self, values)
